/*
 * Copyright 2018 VMware, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Test vertex shader with gl_FrontMaterial and per-vertex materials.
 *
 * When using a VS, per-vertex materials are not used.  Instead, the global
 * material coefficients are the ones accessed via the gl_Front/BackMaterial
 * built-in uniforms.
 */

#include "piglit-util-gl.h"

PIGLIT_GL_TEST_CONFIG_BEGIN
	config.supports_gl_compat_version = 20;
	config.window_visual = PIGLIT_GL_VISUAL_RGBA | PIGLIT_GL_VISUAL_DOUBLE;
	config.khr_no_error_support = PIGLIT_NO_ERRORS;
PIGLIT_GL_TEST_CONFIG_END


/* VS: color = front ambient + front diffuse + two generic attributes */
static const char *vs =
	"attribute vec4 generic1, generic2; \n"
	"void main()\n"
	"{\n"
	"   gl_FrontColor = gl_FrontMaterial.ambient + \n"
	"                   gl_FrontMaterial.diffuse + \n"
	"                   generic1 + generic2; \n"
	"   gl_Position = ftransform(); \n"
	"}\n";

static const char *fs =
	"uniform bool draw_secondary;\n"
	"void main()\n"
	"{\n"
	"   gl_FragColor = gl_Color;\n"
	"}\n";


static int generic1_attr, generic2_attr;


static bool
test(bool use_dlist)
{
	static const float red[] = {1.0, 0.0, 0.0, 1.0};
	static const float green[] = {0.0, 1.0, 0.0, 1.0};
	static const float ambient[] = {0.0, 0.3, 0.5, 1.0};
	static const float diffuse[] = {0.0, 0.2, 0.3, 1.0};
	static const float generic1[] = { 0.2, 0.3, -0.4, 0.0};
	static const float generic2[] = {-0.2, 0.2, -0.4, 0.0};
	static const float pos[4][2] = {
		{ -1, -1 },
		{ 1, -1 },
		{ 1, 1 },
		{ -1, 1 }
	};
	GLuint list = 0;
	bool pass = true;

	glClearColor(0.5, 0.5, 0.5, 0.5);
	glClear(GL_COLOR_BUFFER_BIT);

	if (use_dlist) {
		list = glGenLists(1);
		glNewList(1, GL_COMPILE);
	}

	/* These are the values picked up by gl_FrontMaterial.ambient, diffuse
	 */
	glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);

	glBegin(GL_TRIANGLE_FAN);
	for (int i = 0; i < 4; i++) {
		/* These material values should be ignored */
		glMaterialfv(GL_FRONT, GL_AMBIENT, red);
		glMaterialfv(GL_FRONT, GL_DIFFUSE, red);
		glVertexAttrib4fv(generic1_attr, generic1);
		glVertexAttrib4fv(generic2_attr, generic2);
		glVertex2fv(pos[i]);
	}
	glEnd();

	if (use_dlist) {
		glEndList();
		glCallList(list);
		glDeleteLists(list, 1);
	}

	pass = piglit_probe_rect_rgba(0, 0, piglit_width, piglit_height,
				      green);
	piglit_present_results();

	if (!pass) {
		printf("Fail while testing %s\n",
		       use_dlist ? "display list" : "immediate mode");
	}

	return pass;
}


enum piglit_result
piglit_display(void)
{
	bool pass = test(false);
	pass = test(true) && pass;
	return pass ? PIGLIT_PASS : PIGLIT_FAIL;
}


void
piglit_init(int argc, char **argv)
{
	GLint prog = piglit_build_simple_program(vs, fs);
	glUseProgram(prog);

	generic1_attr = glGetAttribLocation(prog, "generic1");
	generic2_attr = glGetAttribLocation(prog, "generic2");
}
