
bool
check_array_buffer_sub_data(const int ofs, const int length,
		const int expected_data_size, const void *const expected_data);

bool
check_array_buffer_data(const int expected_data_size,
		const void *const expected_data);

void
fill_array_buffer(const int data_size, const void *const data);

