// [config]
// expect_result: pass
// glsl_version: 1.10
// require_extensions: GL_ARB_shading_language_include
// [end config]

#extension GL_ARB_shading_language_include: enable

void main()
{
#line 0 "/shader/./include/../pa th"
}
