
table {
	border: 0pt;
	border-collapse: collapse;
	padding-left: 1.75em;
	padding-right: 1.75em;
	min-width: 100%;
	table-layout: fixed;
}

col:not(:first-child) {
	width: 70pt;
}

tr {
	padding: 4pt;
}

td {
	padding: 4pt;
}

td:first-child {
	padding: 0;
}

td:first-child > div {
	padding: 4pt;
}

.title {
	background-color: #c8c838;
}

.head {
	background-color: #c8c838
}

td.skip, td.warn, td.fail, td.pass, td.trap, td.abort, td.crash, td.dmesg-warn, td.dmesg-fail, td.timeout, td.incomplete, td.notrun {
	text-align: right;
}

td.trap, td.abort, td.crash, td.incomplete {
	color: #ffffff;
}

td.trap a, td.abort a, td.crash a, td.incomplete a {
	color: #ffffff;
}

tr:nth-child(odd)  > td > div.group { background-color: #ffff95 }
tr:nth-child(even) > td > div.group { background-color: #e1e183 }

tr:nth-child(odd)  td.pass  { background-color: #20ff20; }
tr:nth-child(even) td.pass  { background-color: #15e015; }

tr:nth-child(odd)  td.skip  { background-color: #b0b0b0; }
tr:nth-child(even) td.skip  { background-color: #a0a0a0; }
tr:nth-child(odd)  td.notrun  { background-color: #b0b0b0; }
tr:nth-child(even) td.notrun  { background-color: #a0a0a0; }

tr:nth-child(odd)  td.warn  { background-color: #ff9020; }
tr:nth-child(even) td.warn  { background-color: #ef8010; }
tr:nth-child(odd)  td.dmesg-warn  { background-color: #ff9020; }
tr:nth-child(even) td.dmesg-warn  { background-color: #ef8010; }

tr:nth-child(odd)  td.fail  { background-color: #ff2020; }
tr:nth-child(even) td.fail  { background-color: #e00505; }
tr:nth-child(odd)  td.dmesg-fail  { background-color: #ff2020; }
tr:nth-child(even) td.dmesg-fail  { background-color: #e00505; }

tr:nth-child(odd)  td.timeout  { background-color: #83bdf6; }
tr:nth-child(even) td.timeout  { background-color: #4a9ef2; }

tr:nth-child(odd)  td.trap  { background-color: #111111; }
tr:nth-child(even) td.trap  { background-color: #000000; }
tr:nth-child(odd)  td.abort { background-color: #111111; }
tr:nth-child(even) td.abort { background-color: #000000; }
tr:nth-child(odd)  td.crash { background-color: #111111; }
tr:nth-child(even) td.crash { background-color: #000000; }

tr:nth-child(odd)  td.incomplete { background-color: #853385; }
tr:nth-child(even) td.incomplete { background-color: #351435; }
