/*!
[config]
name: Scalar data types declarations
clc_version_min: 10
!*/

kernel void test(global int* out) {
	bool b;
	char c;
	unsigned char uc1; uchar uc2;
	short s;
	unsigned short us1; ushort us2;
	int i;
	unsigned int ui1; uint ui2;
	long l;
	unsigned long ul1; ulong ul2;
	float f;
	size_t st;
	ptrdiff_t pt;
	intptr_t it;
	uintptr_t uit;
	//half h; // Can be only used as a pointer to a buffer

// Needs cl_khr_fp64
#ifdef cl_khr_fp64
#pragma OPENCL EXTENSION cl_khr_fp64 : enable
	double d;
	double* d1;
#endif
}
