// [config]
// expect_result: fail
// glsl_version: 1.10
// require_extensions: GL_ARB_shading_language_include
// dummy_shader_include: True
// [end config]

#include </dummy/path_to/shader_include>

void main()
{
}
