/*
 * Copyright © 2017 Miklós Máté
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/** Paragraph 10 of the Errors section:
 *
 * The error INVALID_OPERATION is generated by AlphaFragmentOp[2..3]ATI
 * if <op> is DOT3_ATI, DOT4_ATI, or DOT2_ADD_ATI and there was no
 * matching ColorFragmentOp[2..3]ATI immediately preceding, or if <op>
 * is not DOT4_ATI and the immediately preceding ColorFragmentOp2ATI
 * specifies an <op> of DOT4_ATI.
 */

#include "piglit-util-gl.h"

PIGLIT_GL_TEST_CONFIG_BEGIN

	config.supports_gl_compat_version = 10;
	config.window_visual = PIGLIT_GL_VISUAL_DOUBLE | PIGLIT_GL_VISUAL_RGBA;

PIGLIT_GL_TEST_CONFIG_END

enum piglit_result
piglit_display(void)
{
	/* UNREACHED */
	return PIGLIT_FAIL;
}

void
piglit_init(int argc, char **argv)
{
	bool pass = true;

	piglit_require_extension("GL_ATI_fragment_shader");

	/* nothing before it */

	/* In these shaders all instructions are invalid, so the shader should
	 * be empty, which is invalid
	 */
	glBeginFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp2ATI(GL_DOT3_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);

	glBeginFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp2ATI(GL_DOT4_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);

	glBeginFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp3ATI(GL_DOT2_ADD_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);

	/* mismatching color op before it */

	glBeginFragmentShaderATI();
	glColorFragmentOp1ATI(GL_MOV_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
			GL_CON_3_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp2ATI(GL_DOT3_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();

	glBeginFragmentShaderATI();
	glColorFragmentOp1ATI(GL_MOV_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
			GL_CON_3_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp2ATI(GL_DOT4_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();

	glBeginFragmentShaderATI();
	glColorFragmentOp1ATI(GL_MOV_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
			GL_CON_3_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp3ATI(GL_DOT2_ADD_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();

	/* after color DOT4 the alpha is not DOT4 */

	glBeginFragmentShaderATI();
	glColorFragmentOp2ATI(GL_DOT4_ATI, GL_REG_0_ATI, GL_NONE, GL_NONE,
			GL_REG_2_ATI, GL_NONE, GL_NONE,
			GL_CON_3_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_NO_ERROR);
	glAlphaFragmentOp2ATI(GL_MUL_ATI, GL_REG_0_ATI, GL_NONE,
			GL_CON_2_ATI, GL_NONE, GL_NONE,
			GL_REG_1_ATI, GL_NONE, GL_NONE);
	pass &= piglit_check_gl_error(GL_INVALID_OPERATION);
	glEndFragmentShaderATI();

	piglit_report_result(pass ? PIGLIT_PASS : PIGLIT_FAIL);
}
