/*
 * Copyright © 2012 VMware, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice (including the next
 * paragraph) shall be included in all copies or substantial portions of the
 * Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

/**
 * Test that texture memory is initialized to a constant color and not
 * stale data that may show old contents of VRAM.
 *
 * To pass this test an OpenGL implementation should initialize the
 * contents of the new buffer to some fixed value (like all zeros).
 * But since that's not spec'd by OpenGL, we only return WARN instead of
 * FAIL if that's not the case.
 *
 * Brian Paul
 * June 2012
 */

#include "piglit-util-gl.h"

PIGLIT_GL_TEST_CONFIG_BEGIN

	config.supports_gl_compat_version = 10;

	config.window_width = 512;
	config.window_height = 512;
	config.window_visual = PIGLIT_GL_VISUAL_RGB | PIGLIT_GL_VISUAL_DOUBLE;

PIGLIT_GL_TEST_CONFIG_END


enum piglit_result
piglit_display(void)
{
	GLfloat firstPixel[4];
	bool pass = true;

        /* init color buffer to red */
	glClearColor(1, 0, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT);

	glEnable(GL_TEXTURE_2D);

        piglit_draw_rect_tex(0, 0, piglit_width, piglit_height,
                             0.0, 0.0, 1.0, 1.0);

	/* The whole image should be same color */
	glReadPixels(0, 0, 1, 1, GL_RGBA, GL_FLOAT, firstPixel);
	pass = piglit_probe_rect_rgb(0, 0, piglit_width, piglit_height,
				     firstPixel);

	piglit_present_results();

	return pass ? PIGLIT_PASS : PIGLIT_WARN;
}


void
piglit_init(int argc, char **argv)
{
        GLsizei width = 1024, height = 1024;
	GLuint tex;

        /* Create texture image with pixels=NULL (undefined) */
	glGenTextures(1, &tex);
	glBindTexture(GL_TEXTURE_2D, tex);
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0,
                     GL_RGBA, GL_UNSIGNED_BYTE, NULL);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
        glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);

	piglit_ortho_projection(piglit_width, piglit_height, GL_FALSE);
}
